import matplotlib.pyplot as plt
import pandas as pd

class ViewContract:

    def __init__(self,view,*args,**kwargs):
        self.view = view

    def add_axes(self,*args,**kwargs):
        return self.view.add_axes(*args,**kwargs)

    def map(self,df: pd.DataFrame):
        self._map(df)
        self.view.map(df)

    def update(self):
        self._update()
        self.view.update()

    def finish(self):
        self._finish()
        self.view.finish()

    def set_default_df(self, df):
        self._set_default_df(df)
        self.view.set_default_df(df)

    def get_axes_of_click(self,event):
        in_axes = self._is_in_axes(event)
        if in_axes is None:
            return self.view.get_axes_of_click(event)
        else:
            return in_axes

    def update_selection(self,series):
        self._update_selection(series)
        self.view.update_selection(series)

    def get_figure(self) -> plt.Figure:
        return self.view.get_figure()

    def set_figure(self,fig):
        self.view.set_figure(fig)

    def configure(self):
        self.view.configure()
        self._configure()

    def get_extent(self):
        extent = self._get_extent()
        if extent is None:
            return self.view.get_extent()
        return extent

    def zoom(self,**kwargs):
        self._zoom(**kwargs)
        self.view.zoom(**kwargs)

    def _finish(self):
        pass

    def _update_selection(self,series):
        pass

    def _get_extent(self):
        return None

    def _map(self,df):
        pass

    def _configure(self):
        pass

    def _update(self):
        """
        do any self updating of views. DONT USE THIS to call flush or draw
        :return:
        """
        pass

    def get_axes(self,*args,**kwargs) -> plt.Axes:
        """
        allows inherited objects to get an axis instance from the main figure

        see: https://matplotlib.org/api/_as_gen/matplotlib.figure.Figure.html for args & kwargs
        :param args: 
        :param kwargs:

        :return: a matplotlib axis instance
        """
        return self.view.get_axes(*args,**kwargs)

    def _is_in_axes(self, event):
        pass

    def _set_default_df(self, df):
        pass

    def _zoom(self,**kwargs):
        pass

class BaseView(ViewContract):

    def __init__(self,dimensions):
        super().__init__(None)


    def configure(self):
        pass

    def update(self):
        self.fig.canvas.draw()
        self.fig.canvas.flush_events()

    def add_axes(self, *args, **kwargs):
        return self.fig.add_axes(*args, **kwargs)

    def get_figure(self):
        return self.fig

    def set_figure(self,fig):
        self.fig = fig

    def map(self,df):
        pass

    def zoom(self,**kwargs):
        pass

    def finish(self):
        self.fig.canvas.draw()

    def update_selection(self, series):
        self.update()

    def is_in_axes(self, event):
        return 'background'

    def set_default_df(self, df):
        pass



from .database_model import DatabaseModel
import itertools
from tkinter import filedialog
import tkinter as tk
import pandas as pd

class ModelController:

    def __init__(self,mt_facade=None, working_directory=None,source_directory=None,**kwargs):
        self.database_model    = DatabaseModel(mt_facade,source_directory)
        self._survey_cycler    = None
        self._selection_cycler = None

    def set_database_directory(self,directory):
        self.database_model.set_directory(directory)

    def add_new_edi_folder(self):
        root = tk.Tk()
        root.withdraw()
        file_path = filedialog.askdirectory(title='select .edi database directory')
        self.database_model.add_new_edi_folder(file_path)

    def save(self):
        root = tk.Tk()
        root.withdraw()
        file_path = filedialog.asksaveasfilename(title="Select .csv save location")
        self.database_model.save(file_path)

    def load(self):
        root = tk.Tk()
        root.withdraw()

        file_path = filedialog.askopenfilename(title="Select .csv database")
        self.database_model.load(file_path)

    def new_df_load(self):
        root = tk.Tk()
        root.withdraw()
        file_path = filedialog.askdirectory(title="Select .edi database directory")
        self.database_model.new_df_load(file_path)

    def get_mapping_data(self)-> pd.DataFrame:
        return self.database_model.get_mapping_data()

    def next_selection(self):
        if self._survey_cycler is None:
            self.create_survey_cycler()

        if self._selection_cycler is None:
            self.create_selection_cycler()
        self._selection_cycler.next()

    def next_survey(self):
        if self._survey_cycler is None:
            self.create_survey_cycler()
        self._survey_cycler.next()
        self.create_selection_cycler()

    def create_survey_cycler(self):
        unique_surveys      = self.database_model.get_unique_surveys()
        self._survey_cycler = ListCycler(unique_surveys)

    def create_selection_cycler(self,extent=None):
        if extent is None:
            survey = self._survey_cycler.get_selected()
            survey_indices = self.database_model.get_survey_indices(survey)
            self._selection_cycler = ListCycler(survey_indices)
        else:
            station_indices = self.database_model.get_indices_matching_extent(extent)
            self._selection_cycler = ListCycler(station_indices)


    def get_selection(self):
        selected_index = self._selection_cycler.get_selected()
        return self.database_model.get_series_at_index(selected_index)


    def update_selection(self,value):
        selected_index = self._selection_cycler.get_selected()
        self.database_model.set_include(selected_index,value)


    def update_cycle_selection(self,value):
        indices = self._selection_cycler.list
        print('selection {}'.format(indices))
        self.database_model.set_include(indices,value)


class ListCycler:
    def __init__(self, lt):
        self.list = lt
        self.generator = self.cycle()
        self.selected  = next(self.generator)

    def cycle(self):
        if self.is_empty():
            for row in itertools.cycle(self.get_iter()):
                yield self.row_ops(row)
        else:
            while True:
                yield None


    def get_iter(self):
        return self.list

    def is_empty(self):
        return self.list

    def row_ops(self,row):
        return row

    def next(self):
        self.selected = next(self.generator)

    def get_selected(self):
        return self.selected



from mtexplore.view.view_base import BaseView
from mtexplore.view.map_view import MapView
from mtexplore.view.mt_view import MTView

base_view_dimensions = (15,10)

def get_view_stack():
    return MTView(
            MapView(
                BaseView(base_view_dimensions)))

class MainView:

    def __init__(self):
        self.view  = get_view_stack()

    def get_view(self):
        return self.view

    def start(self):
        self.view.configure()
        self.view.update()

    def set_figure(self,fig):
        self.view.set_figure(fig)

    def finish(self):
        self.view.finish()

    def update(self):
        self.view.update()


from controller.controller_abstracts import ControllerInterface
from view.view_base import ViewContract
from model.modelcontroller import DatabaseModel

class SaveController(ControllerInterface):

    def __init__(self,controller):
        super().__init__(controller)

    def _key_press_event(self,event):
        if event.key=='j':
            self.model.save()
        elif event.key=='k':
            self.model.load()
            data = self.model.get_mapping_data()
            self.view.map(data)
        elif event.key=='l':
            self.model.new_df_load()
            data = self.model.get_mapping_data()
            self.view.map(data)
        elif event.key==';':
            self.model.add_new_edi_folder()
            data = self.model.get_mapping_data()
            self.view.map(data)
        elif event.key=='h':
            title = "Save & Load Functionality"
            functionality = {
                'Save': 'press j',
                'Load from previous .csv': 'press k',
                'Load from database/*.edi\'s': 'press l',
                'Add new .edi files from folder' : 'press ;'
            }
            self.create_help_menu_string(title, functionality)




from .map_controller import MapController
from .cycle_controller import CycleController
from .save_controller import SaveController
from .scroll_controller import ScrollController
from mtexplore.view.view import MainView
from .controller_abstracts import ControllerBase
from .selection_controller import SelectionController
import matplotlib.pyplot as plt
from matplotlib import rcParams

class MainController:
    dimensions = (15,10)
    def __init__(self):
        self._print_default_bindings()
        self.fig = plt.figure(figsize=self.dimensions)

        self.bind_methods()

        self.bind_to_mlp()

        self.fig.show()
        self.controller = ScrollController(
                            SelectionController(
                                SaveController(
                                    CycleController(
                                        MapController(
                                            ControllerBase())))))

    def bind_to_mlp(self):
        self.fig.canvas.mpl_connect('scroll_event', self.fig._scroll_event_for_mtpy)
        self.fig.canvas.mpl_connect('key_press_event', self.fig._key_press_method_for_mtpy)
        self.fig.canvas.mpl_connect('key_release_event', self.fig._key_release_method_for_mtpy)
        self.fig.canvas.mpl_connect('button_press_event', self.fig._button_press_method_for_mtpy)
        self.fig.canvas.mpl_connect('button_release_event', self.fig._button_release_method_for_mtpy)

    def bind_methods(self):
        self.fig._key_press_method_for_mtpy = self.key_press_events
        self.fig._key_release_method_for_mtpy = self.key_release_events
        self.fig._button_press_method_for_mtpy = self.button_press_events
        self.fig._button_release_method_for_mtpy = self.button_release_events
        self.fig._scroll_event_for_mtpy = self.scroll_event

    def get_figure(self):
        return self.fig

    def add_model(self,model):
        self.controller.connect_model(model)

    def add_view_base(self, view: MainView):
        self.view = view
        self.view.set_figure(self.fig)
        self.controller.connect_view(view.get_view())

    def start(self):
        self.view.start()
        self.controller.update_map()
        self.view.finish()

    def key_press_events(self,event):
        self.controller.key_press_event(event)
        self.view.update()

    def key_release_events(self,event):
        self.controller.key_release_event(event)
        self.view.update()

    def button_press_events(self, event):
        self.controller.button_press_event(event)
        self.view.update()

    def button_release_events(self, event):
        self.controller.button_release_event(event)
        self.view.update()

    def scroll_event(self,event):
        self.controller.scroll_event(event)
        self.view.update()

    def update(self):
        self.controller.update()

    def _print_default_bindings(self):
        rc_keys = ['keymap.fullscreen','keymap.home','keymap.back','keymap.forward','keymap.pan',
                   'keymap.zoom','keymap.save','keymap.quit','keymap.grid','keymap.yscale','keymap.xscale']
        for rc_key in rc_keys:
            try:
                rcParams[rc_key]=[]
            except ValueError:
                print('key {} not in list'.format(rc_key))

    def set_default_frame(self):
        self.controller.set_default_frame()
        self.view.update()





from controller.controller_abstracts import ControllerInterface

class CycleController(ControllerInterface):
    selection_distance = (0.01,0.01)
    def __init__(self,controller):
        super().__init__(controller)

    def _key_press_event(self,event):
        if event.key=='n':
            self.model.next_selection()
            self._update_selection()

        if event.key=='m':
            self.model.next_survey()
            self._update_selection()

        if event.key=='h':
            title = "Cycle Selection Functionality"
            functionality = {
                'next station in group':'press n',
                'next survey group':'press m'
            }
            self.create_help_menu_string(title, functionality)

    def _update_selection(self):
        selection = self.model.get_selection()
        self.view.update_selection(selection)




from controller.controller_abstracts import ControllerInterface

class SelectionController(ControllerInterface):
    def __init__(self,controller):
        super().__init__(controller)

    def _key_press_event(self, event):

        if event.key=='i':
            self.model.update_selection(1)
            self.remap()
        elif event.key=='I':
            self.model.update_cycle_selection(1)
            self.remap()
        elif event.key=='o':
            self.model.update_selection(0)
            self.remap()
        elif event.key=='O':
            self.model.update_cycle_selection(0)
            self.remap()

        elif event.key=='h':
            title = "include/exclude Functionality"
            functionality = {
                'include selected station': 'press i',
                'exclude selected station': 'press o',
                'hold shift to apply en-batch':'press \'shift\''
            }
            self.create_help_menu_string(title, functionality)

    def _key_release_event(self,event):
        pass

    def remap(self):
        data = self.model.get_mapping_data()
        self.view.map(data)





from controller.controller_abstracts import ControllerInterface

class ScrollController(ControllerInterface):
    def __init__(self,controller):
        super().__init__(controller)

    def _set_default_frame(self):
        df= self.model.get_mapping_data()
        self.view.set_default_df(df)

    def _scroll_event(self, event):
        axes = self.view.get_axes_of_click(event)

        if axes=='map':
            center = [event.xdata, event.ydata]
            if event.button=='up':
                self.view.zoom(center=center,zoom='in')

            elif event.button=='down':
                self.view.zoom(center=center,zoom='out')

            else:
                print(event.button)

    def _key_press_event(self,event):

        if event.key=='h':
            title = "include/exclude Functionality"
            functionality = {
                'zoom in scroll': 'scroll up',
                'zoom out scroll': 'scroll down',
                're-center':'c'
            }
            self.create_help_menu_string(title, functionality)

        elif event.key=='c':
            self.view.zoom()





import numpy as np
from controller.controller_abstracts import ControllerInterface

class MapController(ControllerInterface):

    click_range = 1/100

    def __init__(self,controller):
        super().__init__(controller)
        self.pressed  = False
        self.location = None

    def _key_press_event(self,event):
        if event.key=='h':
            title = "Map Selection Functionality"
            functionality = {
                'select individual station': 'click',
                'select station group': 'click and drag'
            }
            self.create_help_menu_string(title, functionality)

    def _update(self):
        data = self.model.get_mapping_data()
        self.view.map(data)

    def _button_release_event(self, event):
        axes_label = self.view.get_axes_of_click(event)

        if axes_label == 'map' and self.pressed:

            lat, lon = event.ydata, event.xdata
            other_loc= np.asarray((lat, lon))
            norm = np.linalg.norm(self.location-other_loc)

            if norm > 0.2:
                extent = {'latitude':  [self.location[0],lat],
                          'longitude':[self.location[1],lon]
                          }
                self.model.create_selection_cycler(extent=extent)
                selection = self.model.get_selection()
                self.set_selection(selection)

            else:
                radius = self.get_radius()
                extent = {'latitude':  [lat-radius, lat+radius],
                          'longitude': [lon-radius, lon+radius]
                          }

            self.model.create_selection_cycler(extent=extent)
            selection = self.model.get_selection()
            self.set_selection(selection)
            self.location=None
            self.pressed=False

    def set_selection(self, selection):
        if selection is not None:
            self.view.update_selection(selection)

    def get_radius(self):
        extent = self.view.get_extent()
        xlim = extent[0]
        ylim = extent[1]
        xrange = xlim[0] - xlim[1]
        yrange = ylim[0] - ylim[1]
        avg   = (abs(xrange) + abs(yrange))/2.0
        radius = avg * self.click_range
        return radius

    def _button_press_event(self, event):
        axes_label = self.view.get_axes_of_click(event)
        if axes_label == 'map':
            lat, lon = event.ydata, event.xdata
            self.pressed  = True
            self.location = np.asarray((lat,lon))






import glob
import pandas as  pd
import itertools
import os

class DatabaseModel:
    _columns= ['latitude','longitude','elevation','easting','northing','utm zone','rotation angle','station id',
                   'survey','include','file','mt obj']
    extension ='.edi'

    click_precision=(0.05,0.05)

    def __init__(self,mt_facade,source_directory):
        self.mt_facade = mt_facade
        self.source_directory = source_directory
        self._station_df        = None
        self._survey_cycler     = None
        self._selection_cycler  = None

    def set_directory(self,source_directory):
        self.source_directory = source_directory
        self._create_station_df()

    def save(self,file_path):
        if '.csv' not in file_path:
            file_path=file_path+'.csv'
        if self._station_df is not None:
            df = self._station_df.drop(columns=['mt obj','Unnamed: 0'])
            df.to_csv(file_path)

    def add_new_edi_folder(self,file_path):
        if os.path.isdir(file_path):
            new_df = self._create_df(file_path)
        else:
            new_df = self._create_single_row(file_path)

        df= pd.concat([new_df,self._station_df])
        df.sort_values(['latitude', 'longitude'], inplace=True)
        df.reset_index(inplace=True)
        self._station_df=df

    def new_df_load(self,file_path):
        self.source_directory=file_path
        self._create_station_df()

    def load(self,file_path):
        try:
            df   = pd.read_csv(file_path)
            mt_obj_handle = self.mt_facade.get_MT_obj()
            df['mt obj'] = df['file'].apply(lambda x: mt_obj_handle(x))
            self._station_df = df
        except:
            print('something is wrong. could not read df.csv')

    def _create_single_row(self,file):
        mt_obj_handle = self.mt_facade.get_MT_obj()
        mt_obj = mt_obj_handle(file)

        def create_row(mt):
            row = [mt[0].lat, mt[0].lon, mt[0].elev, mt[0].east, mt[0].north, mt[0].utm_zone, mt[0].rotation_angle,
                   mt[0].station,
                   mt[0].survey, 0, mt[1], mt[0]]
            return row

        data = create_row(mt_obj)
        df   = pd.DataFrame(data=data, columns=self._columns)
        df.sort_values(['latitude','longitude'],inplace=True)
        return df

    def _create_df(self, directory):
        files = [f for f in glob.glob(directory + '**/*' + self.extension, recursive=True)]
        mt_obj_handle = self.mt_facade.get_MT_obj()
        mt_obj_list = [(mt_obj_handle(f), f) for f in files]

        def create_row(mt):
            row = [mt[0].lat, mt[0].lon, mt[0].elev, mt[0].east, mt[0].north, mt[0].utm_zone, mt[0].rotation_angle,
                   mt[0].station,
                   mt[0].survey, 0, mt[1], mt[0]]
            return row

        data = [create_row(mt) for mt in mt_obj_list]

        df = pd.DataFrame(data=data, columns=self._columns)
        df.sort_values(['latitude', 'longitude'], inplace=True)
        return df

    def _create_station_df(self):
        df=self._create_df(self.source_directory)
        self._station_df = df


    def get_mapping_data(self):
        """

        Returns
        -------
        pandas dataframe with columns ['latitude', 'longitude', 'survey', 'include']

        """
        if self._station_df is not None:
            return self._station_df[['latitude','longitude','survey','include']]
        return pd.DataFrame(columns=['latitude', 'longitude', 'survey', 'include'])

    def get_unique_surveys(self):
        return self._station_df['survey'].unique().tolist()

    def get_survey_indices(self, survey):
        indices = self._station_df.index[self._station_df['survey']==survey].tolist()
        return indices

    def get_indices_matching_extent(self, extent):
        """

        Parameters
        ----------
        extent: list
            lat_minus, lat_plus, lon_minus, lon_plus

        Returns
        -------

        """
        extent['latitude'].sort()
        extent['longitude'].sort()
        indices = self._station_df.index[   (self._station_df['latitude']  > extent['latitude'][0])
                                          & (self._station_df['latitude']  < extent['latitude'][1])

                                          & (self._station_df['longitude'] > extent['longitude'][0])
                                          & (self._station_df['longitude'] < extent['longitude'][1])]

        return indices.tolist()

    def get_series_at_index(self, index):
        if index is not None:
            return self._station_df.loc[index]
        return None

    def set_include(self, selected_index, value):
        self._station_df.loc[selected_index,'include']=value



import os
import sys
sys.path.append(os.getcwd()+os.sep +'mtexplore')
from model.modelcontroller import ModelController
from model import mt_py_facade
from view.view import MainView
from controller.controller import MainController



class Mt_Ex_Main:

    def __init__(self,debug=False,**kwargs):
        mt_facade=mt_py_facade.MtFacade(debug)
        self.controller = MainController()
        self.model = ModelController(mt_facade=mt_facade, **kwargs)
        self.view  = MainView()
        self.controller.add_view_base(self.view)
        self.controller.add_model(self.model)
        self.controller.start()

    def connect_database(self,database_directory):
        self.model.set_database_directory(database_directory)
        self.controller.set_default_frame()
        self.controller.update()



import random
import numpy as np
class MtFacade:
     def __init__(self,debug):
         self._debug = debug

     def set_debug(self, debug_other):
        self._debug= debug_other

     def get_MT_obj(self):
        print('debug is {}'.format(self._debug))
        if self._debug:
            return MT_debug
        else:
            from mtpy.core.mt import MT
        return MTPYFacade

class testsite:

    survey = 'example'

class MT_debug:

    def __init__(self,dir):
        self.dir = dir
        self.station  = str(random.randint(1,200))
        self.lat = random.random()*10 + 20
        self.lon = random.random()*10 -120
        self.elev = 10
        self.east = 0
        self.north = 0
        self.utm_zone = 0
        self.rotation_angle =0
        self.station = 1
        self.survey = 'test survey'

    def get_frequencies(self):
        """

        Returns
        -------
        frequencies: np.array
            returns an array of periods
        """
        return np.linspace(10,1,num=3)

    def extract_phases(self):
        """

        Returns
        -------
        phase_xx, phase_xy, phase_yx, phase_yy

        """
        phase_xx = np.ones((3,))
        phase_yy = np.ones((3,))*2
        phase_xy = np.ones((3,))*5
        phase_yx = np.ones((3,))*10
        return phase_xx, phase_yy, phase_xy, phase_yx

    def extract_phase_error(self):
        phase_xx_err = np.ones((3,))*0.01
        phase_yy_err = np.ones((3,))*0.01
        phase_xy_err = np.ones((3,))*0.01
        phase_yx_err = np.ones((3,))*0.01
        return phase_xx_err, phase_yy_err, phase_xy_err, phase_yx_err

    def extract_resistivity_error(self):
        res_xx_err = np.ones((3,))*0.01
        res_yy_err = np.ones((3,))*0.01
        res_xy_err = np.ones((3,))*0.01
        res_yx_err = np.ones((3,))*0.01
        return res_xx_err, res_yy_err, res_xy_err, res_yx_err

    def extract_resistivity(self):
        res_xx = np.ones((3,))*1
        res_yy = np.ones((3,))*10
        res_xy = np.ones((3,))*5
        res_yx = np.ones((3,))*100
        return res_xx, res_yy, res_xy, res_yx


class MTPYFacade:

    def __init__(self,dir):
        self.dir = dir
        from mtpy.core.mt import MT
        mt_obj = MT(dir)
        self.mt_obj = mt_obj
        self.lat = mt_obj.lat
        self.lon = mt_obj.lon
        self.elev = mt_obj.elev
        self.east = mt_obj.east
        self.north = mt_obj.north
        self.utm_zone = mt_obj.utm_zone
        self.rotation_angle = mt_obj.rotation_angle
        self.station = mt_obj.station
        try:
            self.survey  = mt_obj.Notes.info_dict['SURVEY']
        except KeyError:
            survey = mt_obj.Notes.info_dict['SURVEY ID']
            if survey=='SoUt':
                self.survey = 'UU:Southern Utah'


    def get_frequencies(self):
        """

        Returns
        -------
        frequencies: np.array
            returns an array of periods
        """
        Z_tensor = self.mt_obj.Z
        frequencies = 1.0 / Z_tensor.freq
        return frequencies

    def clean(self,array):
        new_arr = []
        for arr in array:
            arr[arr==np.inf]=0
            arr[arr==-np.inf]=0
            new_arr.append(arr)
        return new_arr

    def extract_phases(self):
        """

        Returns
        -------
        phases: list
            phase values for the given frequencies
        ordered as xx, yy, xy, yx

        """
        arr = []
        arr.append(self.mt_obj.Z.phase_xx)
        arr.append(self.mt_obj.Z.phase_yy)
        arr.append(self.mt_obj.Z.phase_xy)
        arr.append(self.mt_obj.Z.phase_yx)
        arr = self.clean(arr)
        return arr

    def extract_phase_error(self):
        """

        Returns
        -------
        phase_errors: list
            errors for the given frequencies. ordered as
            xx, yy, xy, yx

        """
        err=[]
        err.append(self.mt_obj.Z.phase_err_xx)
        err.append(self.mt_obj.Z.phase_err_yy)
        err.append(self.mt_obj.Z.phase_err_xy)
        err.append(self.mt_obj.Z.phase_err_yx)
        arr = self.clean(err)
        return arr

    def extract_resistivity_error(self):
        """

        Returns
        -------
        resistivity_errors: list
            errors for the given frequencies. ordered as xx, yy, xy, yx
        """
        err=[]
        err.append(self.mt_obj.Z.res_err_xx)
        err.append(self.mt_obj.Z.res_err_yy)
        err.append(self.mt_obj.Z.res_err_xy)
        err.append(self.mt_obj.Z.res_err_yx)
        arr = self.clean(err)
        return arr

    def extract_resistivity(self):
        """

        Returns
        -------
        resistivity: list
            apparent resistivity for the given frequencies. ordered as xx, yy, xy, yx

        """
        res=[]
        res.append(self.mt_obj.Z.res_xx)
        res.append(self.mt_obj.Z.res_yy)
        res.append(self.mt_obj.Z.res_xy)
        res.append(self.mt_obj.Z.res_yx)
        arr = self.clean(res)
        return res




import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="mtexplore",
    version="0.0.6",
    author="Kevin Mendoza",
    author_email='kevinmendoza@icloud.com',
    description="MT Explore is a cartopy and mtpy based utility for viewing and cleaning Magnetotelluric data",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/El-minadero/mt-explore",
    packages=setuptools.find_packages(),
    install_requires = ['cartopy>=0.17.0',
    'matplotlib>=3.0.3',
    'numpy>=0.16.2',
    'pandas>=0.24.',
    'mtpy>=1.0.1'],
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ],
)
print(setuptools.find_packages())


from view.view_base import ViewContract
import cartopy.crs as ccrs
import cartopy.feature as cfeature
import matplotlib.ticker as mticker
import matplotlib.cm as colormap
import numpy as np
import matplotlib.colors as mplcolors
from cartopy.mpl.gridliner import LONGITUDE_FORMATTER, LATITUDE_FORMATTER
from cartopy.io.img_tiles import Stamen
import collections
class MapView(ViewContract):
    map_position=[0.05,0.5,0.4,0.4]
    base_extent   = [-130,-100,20,50]
    _colormap = colormap.get_cmap('hsv')

    s1 = 20
    s2 = 50
    lw = 0.8
    _none_color='none'
    _include='green'
    _exclude='red'
    _scatter_kwargs = dict(s=s1,linewidths=lw,zorder=3,label=None)
    _selection_kwargs = dict(marker='o',
                            edgecolor='red',
                            s=s2,
                            zorder=5,
                            facecolor='None',
                            linewidths=2)
    _cult_feature_nat_earth = dict(
            category='cultural',
            name='admin_1_states_provinces_lines',
            scale='50m',
            facecolor='none')

    def __init__(self,view):
        super().__init__(view)
        self.selection_id=None
        self.selection_handle=None
        self.scatter_handle = None
        self.legend = None
        self.extent = [] + self.base_extent
        self._surveys=[]
        self.gl = None

    def colormap(self,val):
        rgb = self._colormap(val)
        return mplcolors.to_hex(rgb)

    def _configure(self):
        self.ax = self.add_axes(self.map_position,projection=ccrs.PlateCarree())
        states_provinces = cfeature.NaturalEarthFeature(**self._cult_feature_nat_earth)
        url = 'https://map1c.vis.earthdata.nasa.gov/wmts-geo/wmts.cgi'
        layer = 'ASTER_GDEM_Color_Shaded_Relief'
        self.ax.add_wmts(url, layer,alpha=0.8)
        self.ax.coastlines(resolution='50m',zorder=10)
        self.ax.add_feature(states_provinces, edgecolor='black',zorder=2,linewidth=2)
        self.gl = self.ax.gridlines(crs=ccrs.PlateCarree(), draw_labels=True,
                                    alpha=0.5, linestyle='--', color='black')
        self.gl.xlabels_top = False
        self.gl.xlocator = mticker.AutoLocator()
        self.gl.ylocator = mticker.AutoLocator()
        self.gl.xformatter = LONGITUDE_FORMATTER
        self.gl.yformatter = LATITUDE_FORMATTER

    def _update_selection(self,station_data):
        if self.selection_handle is not None:
            self._erase_selection()

        survey = station_data['survey']
        id     = station_data['station id']

        self.selection_handle = self.ax.scatter(station_data['longitude'],station_data['latitude'],**self._selection_kwargs)
        self.ax.set_title('Survey: {}, Station: {}'.format(survey,id))


    def _erase_selection(self):
        self.selection_handle.remove()
        del self.selection_handle
        self.selection_handle=None

    def _erase_scatter(self):
        self.scatter_handle.remove()
        del self.scatter_handle
        self.scatter_handle=None

    def _is_in_axes(self, event):
        x,y = event.x, event.y
        if event.inaxes == self.ax:
            return 'map'
        return None


    def map(self,df):
        if not df.empty:
            df = self.assign_survey_colors(df)
            if self.scatter_handle is not None:
                self._erase_scatter()
            self.scatter_handle=self.ax.scatter(df['longitude'],df['latitude'],edgecolors=df['color'],c=df['qual color'],**self._scatter_kwargs)
            self._check_legend(df)

    def _get_extent(self):
        ylim = self.ax.get_ylim()
        xlim = self.ax.get_xlim()
        return (xlim, ylim)

    def assign_survey_colors(self,df):
        df_survey = df['survey'].unique()
        color_dict = {}
        for ix, survey in enumerate(df_survey):
            color_dict[survey]= self.colormap(ix/len(df_survey))

        for key,value in color_dict.items():
            df.at[df['survey']==key,'color']=value
        df.at[df['include'] == 0, 'qual color'] = self._none_color
        df.at[df['include'] == 1, 'qual color'] = self._include
        return df

    def _check_legend(self,df):
        items = collections.Counter(df['survey'].unique())
        if self.legend is not None:
            self.legend.remove()
        handles=[]
        for ix, survey in enumerate(items):
            color = self.colormap(ix / len(items))
            handle = self.ax.scatter([], [], label=survey, edgecolors=color, c='none', s=self.s1,linewidths=self.lw)
            handles.append(handle)

        handles.append(self.ax.scatter([], [], label='exclude station',  edgecolors='black',   c=self._none_color))
        handles.append(self.ax.scatter([], [], label='include station', edgecolors='black', c=self._include))
        ncols = int((len(items)+3)/3)
        self.legend=self.ax.legend(handles,items,ncol=ncols,loc='lower left')

    def set_default_extent(self,df):
        extent = [None,None,None,None]
        if df is not None:
            extent[0]=df['longitude'].min()
            extent[1]=df['longitude'].max()
            extent[2]=df['latitude'].min()
            extent[3]=df['latitude'].max()
            x_delta = extent[1] - extent[0]
            y_delta = extent[3] - extent[2]
            x_delta *= 1.1
            y_delta *= 1.1
            x_left = (extent[0] + extent[1]) / 2.0 - x_delta / 2
            x_right= (extent[0] + extent[1]) / 2.0 + x_delta / 2
            y_down = (extent[2] + extent[3]) / 2.0 - y_delta / 2 - y_delta / 3
            y_up   = (extent[2] + extent[3]) / 2.0 + y_delta / 2
            extent =  [x_left, x_right, y_down, y_up]
        self.scale=1.0
        self.extent=extent

    def _set_default_df(self, df):
        self.set_default_extent(df)
        self._update_map_extent()

    def _zoom(self,**kwargs):
        self._update_map_extent(**kwargs)

    def _update_map_extent(self, center=None, zoom=None):
        extent = [x for x in self.extent]
        if center is None:
            self.scale=1.0
            x = (extent[0]+extent[1])/2
            y = (extent[2]+extent[3])/2
            center = [x,y]

        else:
            if zoom=='in':
                self.scale*=0.9
            else:
                self.scale*=1.1

        extent = [x for x in self.extent]
        dx_left  = extent[0] - center[0]
        dx_right = extent[1] - center[0]
        dy_down  = extent[2] - center[1]
        dy_up    = extent[3] - center[1]

        dx_left *=self.scale
        dx_right*=self.scale
        dy_down *=self.scale
        dy_up   *=self.scale

        extent = [dx_left,dx_right,dy_down,dy_up]

        extent[0]+=center[0]
        extent[1]+=center[0]
        extent[2]+=center[1]
        extent[3]+=center[1]

        self.ax.set_extent(extent,crs=ccrs.PlateCarree())
        self.update_latlons(extent)

    def update_latlons(self,extent):
        self.gl.ylocator.view_limits(extent[0],extent[1])
        self.gl.xlocator.view_limits(extent[2],extent[3])
        self.gl.ylocator.refresh()
        self.gl.xlocator.refresh()


    def get_xyq_values_for_survey(self, stations, survey):
        x_values = []
        y_values = []
        q_values = []
        for id, entry in stations[survey].items():
            q_values.append(self.get_color_value(entry[1]))
            x_values.append(entry[0][1])
            y_values.append(entry[0][0])
        return x_values, y_values, q_values

    def get_color_value(self,value):
        if value is None:
            return self._none_color
        elif value=='1':
            return self._passing_color
        elif value=='2':
            return self._marginal_color
        else:
            return self._failing_color




from view.view_base import ViewContract
from model.modelcontroller import ModelController


class ControllerInterface:


    def __init__(self,controller):
        self.controller = controller

    def create_help_menu_string(self,title,functionality):
        separator='***************************\n'

        header  = separator + title + '\n' + separator
        actions = ""
        for key, value in functionality.items():
            actions=actions + '\t' + key + ":" + "\t\t" + value + '\n'

        total = header + actions

        print(total)

    def connect_view(self,view: ViewContract):
        self._connect_view(view)
        self.controller.connect_view(view)

    def connect_model(self, model: ModelController):
        self._connect_model(model)
        self.controller.connect_model(model)

    def update_map(self):
        self._update_map()
        self.controller.update_map()

    def update_em(self):
        self._update_em()
        self.controller.update_em()

    def button_press_event(self,event):
        self._button_press_event(event)
        self.controller.button_press_event(event)

    def button_release_event(self,event):
        self._button_release_event(event)
        self.controller.button_release_event(event)

    def key_press_event(self,event):
        self._key_press_event(event)
        self.controller.key_press_event(event)

    def key_release_event(self, event):
        self._key_release_event(event)
        self.controller.key_release_event(event)

    def scroll_event(self, event):
        self._scroll_event(event)
        self.controller.scroll_event(event)

    def update(self):
        self._update()
        self.controller.update()

    def set_default_frame(self):
        self._set_default_frame()
        self.controller.set_default_frame()

    def _connect_view(self, view: ViewContract):
        self.view = view

    def _connect_model(self, model: ModelController):
        self.model = model

    def _update(self):
        pass

    def _update_map(self):
        pass

    def _update_em(self):
        pass

    def _key_press_event(self,event):
        pass

    def _key_release_event(self,event):
        pass

    def _button_press_event(self, event):
        pass

    def _button_release_event(self, event):
        pass

    def _scroll_event(self,event):
        pass

    def _set_default_frame(self):
        pass


class ControllerBase(ControllerInterface):

    def __init__(self,*args,**kwargs):
        super().__init__(None)

    def connect_view(self, view: ViewContract):
        pass

    def connect_model(self, model: ModelController):
        pass

    def button_press_event(self, event):
        pass
    
    def button_release_event(self,event):
        pass

    def update_map(self):
        pass

    def update_em(self):
        pass

    def scroll_event(self,event):
        pass

    def update(self):
        pass

    def key_press_event(self, event):
        pass

    def key_release_event(self, event):
        pass

    def set_default_frame(self):
        pass


from view.view_base import ViewContract
import numpy as np
from matplotlib.ticker import ScalarFormatter


class MTView(ViewContract):
    app_res_position = [0.55, 0.5, 0.4, 0.4]
    phase_position   = [0.55, 0.25, 0.4, 0.2]

    def __init__(self,view):
        super().__init__(view)
        self.app_res = AppRes()
        self.phase   = Phase()

    def _update_selection(self,selection_row,**kwargs):
        mt_obj = selection_row['mt obj']
        self.phase.update(mt_obj)
        self.app_res.update(mt_obj)

    def _is_in_axes(self, event):
        if event.inaxes==self.app_res.ax:
            return 'res'
        elif event.inaxes==self.phase.axe2:
            return 'phase'
        else:
            return None


    def _configure(self):
        ax1=self.add_axes(self.app_res_position)
        ax2=self.add_axes(self.phase_position)
        ax1.get_shared_x_axes().join(ax1,ax2)
        ax1.set_xlim([1e-3, 1e5])
        ax1.tick_params(axis='x',which='major',  length=10, grid_linewidth=3,grid_linestyle='-')
        ax2.tick_params(axis='x', which='major', length=10, grid_linewidth=3,grid_linestyle='-')
        ax1.tick_params(axis='x', which='minor', length=7, grid_linewidth=3, grid_linestyle='--')
        ax2.tick_params(axis='x', which='minor', length=7, grid_linewidth=3, grid_linestyle='--')
        ax1.get_xaxis().set_major_formatter(ScalarFormatter())
        self.app_res.configure(ax1)
        self.phase.configure(ax2)
        ax1.loglog()


class _Format():
    yx_marker='o'
    xy_marker='s'
    xx_marker='s'
    yy_marker='o'
    marker_size=6
    lw=1.5
    yx_color=(1, 0.7, 0.2)
    xy_color=(0.7, 0.2, 1)
    xx_color=(1,0,0)
    yy_color=(0,0,1)
    yx_ls=':'
    xy_ls=':'
    xx_ls=':'
    yy_ls=':'

    def __init__(self):
        pass

class Phase(_Format):
    fontsize = 12
    weight = 'bold'
    def __init__(self):
        super().__init__()
        self.plot = []
        self.legend = False

    def _to_pi_strings(self,angle_deg):
        multiplier = int(4*angle_deg/180.0)
        if abs(multiplier)==4:
            sign = np.sign(multiplier)
            if sign==1:
                return r'$\pi$'
            else:
                return r'$-\pi$'
        elif multiplier==0:
            return '0'
        elif multiplier<0:
            return r'$-\frac{' + str(abs(multiplier))+'}{4}\pi$'
        else:
            return r'$\frac{' + str(multiplier) + '}{4}\pi$'

    def configure(self,axes):
        axes.set_ylim([-180,180])
        axes.set_yticks(np.arange(-180,180+45,45))
        axes.set_ylabel('Apparent Phase',fontsize=self.fontsize,weight=self.weight)
        axes.set_xlabel('Period', fontsize=self.fontsize, weight=self.weight)
        axes.xaxis.grid(True,linewidth=1,which='both',color='black')
        axes.plot([1e-2,1e6],[1,1],color='orange',linestyle='--')
        axe2 = axes.twinx()
        axe2.yaxis.tick_right()
        ticks = np.arange(-180,180+45,45)
        labels= [self._to_pi_strings(x) for x in ticks]
        axe2.set_yticks(ticks)
        axe2.set_yticklabels(labels)
        self.axe2=axe2
        self.ax =axes

    def update(self,mt_obj,*args,**kwargs):
        if self.plot:
            for plot in self.plot:
                plot.remove()

            self.plot=[]
        frequencies=mt_obj.get_frequencies()

        res_xx,     res_yy,     res_xy,     res_yx     = mt_obj.extract_phases()
        res_xx_err, res_yy_err, res_xy_err, res_yx_err = mt_obj.extract_phase_error()

        #self.ax.set_xlim([log_cast(0.1, min(frequencies[np.nonzero(res_xx)])),
        #                 log_cast(10 , max(frequencies[np.nonzero(res_xx)]))])

        self.plot_series(frequencies, res_xx, res_xx_err,
                         self.xy_marker, self.xx_color,
                         self.xx_color, self.xx_color, self.xx_ls, r'$\rho_{xx}$')

        self.plot_series(frequencies, res_xy, res_xy_err,
                         self.xy_marker, self.xy_color,
                         self.xy_color, self.xy_color, self.xy_ls, r'$\rho_{xy}$')

        self.plot_series(frequencies, res_yy, res_yy_err,
                         self.yy_marker, self.yy_color,
                         self.yy_color, self.yy_color, self.yy_ls, r'$\rho_{yy}$')

        self.plot_series(frequencies, res_yx, res_yx_err,
                         self.yx_marker, self.yx_color,
                         self.yx_color, self.yx_color, self.yx_ls, r'$\rho_{yx}$')

    def plot_series(self, frequencies, res_xx, res_xx_err,
                            marker, mec, color, ecolor, ls, label):

        plot_obj=self.ax.errorbar(get_nonzero_array(frequencies, res_xx),
                                          get_nonzero_array(res_xx, res_xx),
                                          marker=marker,
                                          ms=self.marker_size,
                                          mew=self.lw,
                                          mec=mec,
                                          color=color,
                                          ecolor=ecolor,
                                          ls=ls,
                                          lw=self.lw,
                                          yerr=get_nonzero_array(res_xx_err, res_xx),
                                          capsize=self.marker_size,
                                          capthick=self.lw, label=label)
        self.plot.append(plot_obj)

class AppRes(_Format):
    fontsize = 12
    weight ='bold'

    def __init__(self):
        super().__init__()
        self.plot=[]
        self.legend=False

    def configure(self, axes):
        axes.yaxis.set_major_formatter(ScalarFormatter())
        axes.set_ylim([1e-1,2e4])
        axes.set_ylabel('Apparent Resistivity',fontsize=self.fontsize,weight=self.weight)
        axes.tick_params(axis='both',length=10,grid_linewidth=3)
        axes.tick_params(axis='y', which='minor', length=7, grid_linewidth=0.5, grid_linestyle='--')
        axes.grid(True,linewidth=1,which='both',color='black')
        self.ax = axes

    def update(self,mt_obj,*args,**kwargs):
        if self.plot:
            for plot in self.plot:
                plot.remove()

            self.plot=[]

        frequencies = mt_obj.get_frequencies()

        #self.ax.set_xlim([min(frequencies),max(frequencies)])

        res_xx,     res_yy,     res_xy,     res_yx     = mt_obj.extract_resistivity()
        res_xx_err, res_yy_err, res_xy_err, res_yx_err = mt_obj.extract_resistivity_error()

        max_yval =  get_max_value(res_xx, res_yy, res_xy, res_yx)
        min_yval = -get_max_value(-res_xx, -res_yy, -res_xy, -res_yx)
        self.ax.set_ylim([log_cast(0.1,min_yval),log_cast(10,max_yval)])

        self.plot_series(frequencies, res_xx, res_xx_err,
                         self.xy_marker,self.xx_color,
                         self.xx_color,self.xx_color,self.xx_ls,r'$\rho_{xx}$')

        self.plot_series(frequencies, res_xy, res_xy_err,
                         self.xy_marker, self.xy_color,
                         self.xy_color, self.xy_color, self.xy_ls,r'$\rho_{xy}$')

        self.plot_series(frequencies, res_yy, res_yy_err,
                         self.yy_marker, self.yy_color,
                         self.yy_color, self.yy_color, self.yy_ls,r'$\rho_{yy}$')

        self.plot_series(frequencies, res_yx, res_yx_err,
                         self.yx_marker, self.yx_color,
                         self.yx_color, self.yx_color, self.yx_ls,r'$\rho_{yx}$')

        if not self.legend:
            self.legend=self.ax.legend(loc='upper right',ncol=2)

    def plot_series(self, frequencies, res_xx, res_xx_err,
                            marker, mec, color, ecolor, ls, label):

        plot_obj=self.ax.errorbar(get_nonzero_array(frequencies, res_xx),
                                          get_nonzero_array(res_xx, res_xx),
                                          marker=marker,
                                          ms=self.marker_size,
                                          mew=self.lw,
                                          mec=mec,
                                          color=color,
                                          ecolor=ecolor,
                                          ls=ls,
                                          lw=self.lw,
                                          yerr=get_nonzero_array(res_xx_err, res_xx),
                                          capsize=self.marker_size,
                                          capthick=self.lw, label=label)
        self.plot.append(plot_obj)


def get_max_value(*args):
    nonzero_args=[]
    for arg in args:
        nonzero_args.append(get_nonzero_array(arg,arg))
    newlist=[]
    for arg in nonzero_args:
        newlist.append(max(arg))
    return max(newlist)


def log_cast(multiplier,value):
    return np.power(10.0,int(np.log10(value*multiplier)))


def get_nonzero_array(target, source):
    nonzero_array = np.nonzero(source)
    return target[nonzero_array]
